@@ -43,8 +43,27 @@ def parse_args(args=None):
         groups A and B. As specifial cases, 'A=10` etc refers to cases when
         infection happens from outside of the simulated population (community
         infection), and "*", "?" and "[]" (range) can be used to refer to multiple
-        groups using the same rules as filename expansion, and !name as "not".'''
+        groups using the same rules as filename expansion, and !name as "not".
+        Finally "&" means myself, so "A*-&=10" means vivinity within its all
+        group if 10, and "A*-!&=0" meaning "A*" is isolated and will not infect
+        other groups.'''
     )
+    parser.add_argument(
+        '--immunity-of-recovered',
+        nargs='*',
+        type=float,
+        help='''Specifies the immunity (probability to fend off an infection event)
+            of carries who recovers from an infection. It can be a single number, or
+            two numbers for symptomatic and asyptomatic carriers separately. Default
+            to 0.99.''')
+    parser.add_argument(
+        '--infectivity-of-recovered',
+        nargs='*',
+        type=float,
+        help='''Specifies the infectivity (reduction of viral load for recovered
+            individuals once they have infected). It can be a single number, or
+            two numbers for symptomatic and asyptomatic carriers separately. Default
+            to 0.99.''')
     parser.add_argument(
         '--susceptibility',
         nargs='+',
@@ -74,8 +93,7 @@ def parse_args(args=None):
         nargs='+',
         default=['piecewise'],
         help='''Model used for asymptomatic cases with parameters. The default model
-            normal has a duration of 8 days after incubation, and a peak happens at 2/3 of
-            incubation. An alternative piece wise model has a stage of no infectivity, a stage
+            piecewise has a stage of no infectivity, a stage
             for increasing and a stage for decreasing infectivity. The proportion of stages
             and time after incubation period are controlled by a lognormal distribution.
             Parameters to this model could be specified but not recommended.
@@ -122,10 +140,17 @@ def parse_args(args=None):
             it recovers). "reintegrate" (remove from quarantine, which is counterintuitive but
             can be used for selecting uninfected indivudal through quarantine). All options can
             be followed by a "proportion" in the format of "remove?proportion=0.7", and quarantine can be
-            specified as "quarantine?duration=7" etc to specify duration of quarantine. If multiple
+            specified as "quarantine?duration=7" etc to specify duration of quarantine. "tracing=XX"
+            can be used to indicate that contact tracing will be performed for positive individuals,
+            with XX being the success rate for contact tracing, If multiple
             subpopulation will have different ways to handle symptomatic cases, they should be specified
             as multipliers (e.g. A=remove B=quarantine). Default to  "remove", meaning all symptomatic cases will be
             removed from population.''')
+    parser.add_argument(
+        '--handle-infection',
+        help='''How infections are handled. Currently it only accepts ignore=t/7<2 which basically means infection during the
+            first two days are ignored (weekend).'''
+        )
     parser.add_argument(
         '--infectors',
         nargs='*',
@@ -210,14 +235,21 @@ class FilteredStringIO(StringIO):
     def __init__(self, track_events=None):
         super().__init__()
         self._track_events = track_events
+        self._track_plugins = set()
         if self._track_events is not None:
-            self._track_events = set(self._track_events)
+            self._track_plugins = set([x.split('.')[-1] for x in self._track_events if x.startswith('PLUGIN')])
+            self._track_events = set([x.split('.')[0] for x in self._track_events if not x.startswith('PLUGIN')])
             for evt in ('START', 'ERROR', 'END'):
                 if evt not in self._track_events:
                     self._track_events.add(evt)
 
     def write(self, text):
-        if not self._track_events or text.split('\t')[1] in self._track_events:
+        _, evt, _, params = text.split('\t')
+        if evt == 'PLUGIN':
+            # output all plugins
+            if 'PLUGIN' in self._track_plugins or params.split(',')[0][5:] in self._track_plugins:
+                super().write(text)
+        elif not self._track_events or evt in self._track_events:
             super().write(text)
 
 
@@ -275,6 +307,10 @@ def main(argv=None):
         print(f'COVID10 Outbreak Simulator version {__version__}')
         sys.exit(0)
 
+    if args.logfile and '/' in args.logfile:
+        dirname = os.path.dirname(args.logfile)
+        os.makedirs(dirname, exist_ok=True)
+
     plugins = []
     if args.plugin:
         if args.plugin[0] == '-h':
