@@ -102,6 +102,35 @@ def test_draw_prop_asym_carriers(default_model):
     assert math.fabs(
         sum(props) / 1000 - default_model.params.prop_asym_carriers_loc) < 0.05
 
+def test_immunity_of_recovered(default_model):
+    assert default_model.params.immunity_of_recovered[0] == 0.99
+    assert default_model.params.immunity_of_recovered[1] == 0.99
+
+    default_model.params.set_immunity_of_recovered([0.8])
+
+    assert default_model.params.immunity_of_recovered[0] == 0.80
+    assert default_model.params.immunity_of_recovered[1] == 0.80
+
+    default_model.params.set_immunity_of_recovered([0.7, 0.6])
+
+    assert default_model.params.immunity_of_recovered[0] == 0.7
+    assert default_model.params.immunity_of_recovered[1] == 0.6
+
+
+def test_infectivity_of_recovered(default_model):
+    assert default_model.params.infectivity_of_recovered[0] == 0.50
+    assert default_model.params.infectivity_of_recovered[1] == 0.50
+
+    default_model.params.set_infectivity_of_recovered([0.2])
+
+    assert default_model.params.infectivity_of_recovered[0] == 0.20
+    assert default_model.params.infectivity_of_recovered[1] == 0.20
+
+    default_model.params.set_infectivity_of_recovered([0.3, 0.1])
+
+    assert default_model.params.infectivity_of_recovered[0] == 0.3
+    assert default_model.params.infectivity_of_recovered[1] == 0.1
+
 
 def test_drawn_is_asymptomatic(default_model):
     prop = default_model.draw_prop_asym_carriers()
@@ -140,37 +169,6 @@ def draw_random_incubation_period(default_model):
         scale=default_model.params.incubation_period_sigma) < 0.001
 
 
-def test_get_normal_symptomatic_transmission_probability(default_model):
-
-    incu = default_model.draw_random_incubation_period()
-    R0 = default_model.draw_random_r0(symptomatic=True)
-
-    N = 10000
-    r = []
-    for i in range(N):
-        x_grid, prob = default_model.get_symptomatic_transmission_probability(
-            incu, R0, default_model.draw_infection_params(symptomatic=True))
-        infected = np.random.binomial(1, prob, len(x_grid))
-        r.append(sum(infected))
-    #
-    assert math.fabs(sum(r) / N) - R0 < 0.05
-
-
-def test_get_normal_asymptomatic_transmission_probability(default_model):
-
-    R0 = default_model.draw_random_r0(symptomatic=False)
-
-    N = 10000
-    r = []
-    for i in range(N):
-        x_grid, prob = default_model.get_asymptomatic_transmission_probability(
-            R0, default_model.draw_infection_params(symptomatic=False))
-        infected = np.random.binomial(1, prob, len(x_grid))
-        r.append(sum(infected))
-    #
-    assert math.fabs(sum(r) / N) - R0 < 0.05
-
-
 
 def test_get_piecewise_symptomatic_transmission_probability(default_model):
 
